/* 
 * Copyright (c) 2001, 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: runtime.h,v 1.2 2002/02/16 12:09:14 malekith Exp $
 */

#include <stddef.h>

/*** Basic types. ***/

/* Integer of the same size, as pointer. */
#define ptroff long



/*** Memory management. ***/

/* All gont_xmalloc* routines prints error message and aborts if there
 * is not enough memory to satisfy request. */

/* Allocate given amount of memory. */
void *gont_xmalloc(ptroff s);

/* Allocate given amount of memory and clear it. */
void *gont_xmalloc0(ptroff s);

/* Allocate given amount of memory, one cannot put pointers into this memory 
 * then (for example this is used for strings). */
void *gont_xmalloc_atomic(ptroff s);

/* As above, and clear it. */
void *gont_xmalloc0_atomic(ptroff s);



/*** Exceptions. ***/
struct gont_try_block;		/* Hidden. */

/* Value returned from this function can be used as __builtin_setjmp 
 * buffer. */
struct gont_try_block *gont_gont_try_block(void);

/* Pop value from try-block chain. */
void *gont_pop_raised_value(void);

/* Raise exception. */
void gont_raise(void *v);



/*** Complex Gont datatypes. ***/

/* Lambda expressions. */
struct gont_lambda_exp {
	void *fnc;		/* Pointer to function. */
	void *closure;		/* Pointer to its closure. */
};

/* Make new lambda expression. */
struct gont_lambda_exp *gont_make_lambda(void *fnc, void *closure);


/* Strings. */
struct gont_string {
	ptroff len;
	char *data;
};

struct gont_string *gont_make_string(ptroff size, char *content);


/* Floats. */
double *gont_make_float(double v);


/* Unions. */
struct gont_union {
	ptroff field_no;
	void *value;
};

struct gont_union *gont_make_union(ptroff field_no, void *value);


/* EOF */
